@@ -2,8 +2,6 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-// @dart = 2.6
-
 part of dart.io;
 
 // These match enum StdioHandleType in file.h
@@ -19,46 +17,44 @@ class _StdStream extends Stream<List<int>> {
 
   _StdStream(this._stream);
 
-  StreamSubscription<List<int>> listen(void onData(List<int> event),
-      {Function onError, void onDone(), bool cancelOnError}) {
+  StreamSubscription<List<int>> listen(void onData(List<int> event)?,
+      {Function? onError, void onDone()?, bool? cancelOnError}) {
     return _stream.listen(onData,
         onError: onError, onDone: onDone, cancelOnError: cancelOnError);
   }
 }
 
-/**
- * [Stdin] allows both synchronous and asynchronous reads from the standard
- * input stream.
- *
- * Mixing synchronous and asynchronous reads is undefined.
- */
+/// The standard input stream of the process.
+///
+/// Allows both synchronous and asynchronous reads from the standard
+/// input stream.
+///
+/// Mixing synchronous and asynchronous reads is undefined.
 class Stdin extends _StdStream implements Stream<List<int>> {
   int _fd;
 
   Stdin._(Stream<List<int>> stream, this._fd) : super(stream);
 
-  /**
-   * Read a line from stdin.
-   *
-   * Blocks until a full line is available.
-   *
-   * Lines my be terminated by either `<CR><LF>` or `<LF>`. On Windows in cases
-   * where the [stdioType] of stdin is [StdioType.terminal] the terminator may
-   * also be a single `<CR>`.
-   *
-   * Input bytes are converted to a string by [encoding].
-   * If [encoding] is omitted, it defaults to [systemEncoding].
-   *
-   * If [retainNewlines] is `false`, the returned String will not include the
-   * final line terminator. If `true`, the returned String will include the line
-   * terminator. Default is `false`.
-   *
-   * If end-of-file is reached after any bytes have been read from stdin,
-   * that data is returned without a line terminator.
-   * Returns `null` if no bytes preceded the end of input.
-   */
-  String readLineSync(
-      {Encoding encoding: systemEncoding, bool retainNewlines: false}) {
+  /// Reads a line from stdin.
+  ///
+  /// Blocks until a full line is available.
+  ///
+  /// Lines my be terminated by either `<CR><LF>` or `<LF>`. On Windows,
+  /// in cases where the [stdioType] of stdin is [StdioType.terminal],
+  /// the terminator may also be a single `<CR>`.
+  ///
+  /// Input bytes are converted to a string by [encoding].
+  /// If [encoding] is omitted, it defaults to [systemEncoding].
+  ///
+  /// If [retainNewlines] is `false`, the returned string will not include the
+  /// final line terminator. If `true`, the returned string will include the line
+  /// terminator. Default is `false`.
+  ///
+  /// If end-of-file is reached after any bytes have been read from stdin,
+  /// that data is returned without a line terminator.
+  /// Returns `null` if no bytes preceded the end of input.
+  String? readLineSync(
+      {Encoding encoding = systemEncoding, bool retainNewlines = false}) {
     const CR = 13;
     const LF = 10;
     final List<int> line = <int>[];
@@ -114,74 +110,57 @@ class Stdin extends _StdStream implements Stream<List<int>> {
     return encoding.decode(line);
   }
 
-  /**
-   * Check if echo mode is enabled on [stdin].
-   */
+  /// Whether echo mode is enabled on [stdin].
+  ///
+  /// If disabled, input from to console will not be echoed.
+  ///
+  /// Default depends on the parent process, but is usually enabled.
+  ///
+  /// On Windows this mode can only be enabled if [lineMode] is enabled as well.
   external bool get echoMode;
+  external set echoMode(bool echoMode);
 
-  /**
-   * Enable or disable echo mode on [stdin].
-   *
-   * If disabled, input from to console will not be echoed.
-   *
-   * Default depends on the parent process, but usually enabled.
-   *
-   * On Windows this mode can only be enabled if [lineMode] is enabled as well.
-   */
-  external void set echoMode(bool enabled);
-
-  /**
-   * Check if line mode is enabled on [stdin].
-   */
+  /// Whether line mode is enabled on [stdin].
+  ///
+  /// If enabled, characters are delayed until a newline character is entered.
+  /// If disabled, characters will be available as typed.
+  ///
+  /// Default depends on the parent process, but is usually enabled.
+  ///
+  /// On Windows this mode can only be disabled if [echoMode] is disabled as well.
   external bool get lineMode;
+  external set lineMode(bool lineMode);
 
-  /**
-   * Enable or disable line mode on [stdin].
-   *
-   * If enabled, characters are delayed until a new-line character is entered.
-   * If disabled, characters will be available as typed.
-   *
-   * Default depends on the parent process, but usually enabled.
-   *
-   * On Windows this mode can only be disabled if [echoMode] is disabled as well.
-   */
-  external void set lineMode(bool enabled);
-
-  /**
-    * Whether connected to a terminal that supports ANSI escape sequences.
-    *
-    * Not all terminals are recognized, and not all recognized terminals can
-    * report whether they support ANSI escape sequences, so this value is a
-    * best-effort attempt at detecting the support.
-    *
-    * The actual escape sequence support may differ between terminals,
-    * with some terminals supporting more escape sequences than others,
-    * and some terminals even differing in behavior for the same escape
-    * sequence.
-    *
-    * The ANSI color selection is generally supported.
-    *
-    * Currently, a `TERM` environment variable containing the string `xterm`
-    * will be taken as evidence that ANSI escape sequences are supported.
-    * On Windows, only versions of Windows 10 after v.1511
-    * ("TH2", OS build 10586) will be detected as supporting the output of
-    * ANSI escape sequences, and only versions after v.1607 ("Anniversary
-    * Update", OS build 14393) will be detected as supporting the input of
-    * ANSI escape sequences.
-    */
+  /// Whether connected to a terminal that supports ANSI escape sequences.
+  ///
+  /// Not all terminals are recognized, and not all recognized terminals can
+  /// report whether they support ANSI escape sequences, so this value is a
+  /// best-effort attempt at detecting the support.
+  ///
+  /// The actual escape sequence support may differ between terminals,
+  /// with some terminals supporting more escape sequences than others,
+  /// and some terminals even differing in behavior for the same escape
+  /// sequence.
+  ///
+  /// The ANSI color selection is generally supported.
+  ///
+  /// Currently, a `TERM` environment variable containing the string `xterm`
+  /// will be taken as evidence that ANSI escape sequences are supported.
+  /// On Windows, only versions of Windows 10 after v.1511
+  /// ("TH2", OS build 10586) will be detected as supporting the output of
+  /// ANSI escape sequences, and only versions after v.1607 ("Anniversary
+  /// Update", OS build 14393) will be detected as supporting the input of
+  /// ANSI escape sequences.
   external bool get supportsAnsiEscapes;
 
-  /**
-   * Synchronously read a byte from stdin. This call will block until a byte is
-   * available.
-   *
-   * If at end of file, -1 is returned.
-   */
+  /// Synchronously reads a byte from stdin.
+  ///
+  /// This call will block until a byte is available.
+  ///
+  /// If at end of file, -1 is returned.
   external int readByteSync();
 
-  /**
-   * Returns true if there is a terminal attached to stdin.
-   */
+  /// Whether there is a terminal attached to stdin.
   bool get hasTerminal {
     try {
       return stdioType(this) == StdioType.terminal;
@@ -194,73 +173,63 @@ class Stdin extends _StdStream implements Stream<List<int>> {
   }
 }
 
-/**
- * [Stdout] represents the [IOSink] for either `stdout` or `stderr`.
- *
- * It provides a *blocking* `IOSink`, so using this to write will block until
- * the output is written.
- *
- * In some situations this blocking behavior is undesirable as it does not
- * provide the same non-blocking behavior as dart:io in general exposes.
- * Use the property [nonBlocking] to get an `IOSink` which has the non-blocking
- * behavior.
- *
- * This class can also be used to check whether `stdout` or `stderr` is
- * connected to a terminal and query some terminal properties.
- *
- * The [addError] API is inherited from  [StreamSink] and calling it will result
- * in an unhandled asynchronous error unless there is an error handler on
- * [done].
- */
+/// An [IOSink] connected to either the standard out or error of the process.
+///
+/// Provides a *blocking* `IOSink`, so using it to write will block until
+/// the output is written.
+///
+/// In some situations this blocking behavior is undesirable as it does not
+/// provide the same non-blocking behavior that `dart:io` in general exposes.
+/// Use the property [nonBlocking] to get an [IOSink] which has the non-blocking
+/// behavior.
+///
+/// This class can also be used to check whether `stdout` or `stderr` is
+/// connected to a terminal and query some terminal properties.
+///
+/// The [addError] API is inherited from [StreamSink] and calling it will result
+/// in an unhandled asynchronous error unless there is an error handler on
+/// [done].
 class Stdout extends _StdSink implements IOSink {
   final int _fd;
-  IOSink _nonBlocking;
+  IOSink? _nonBlocking;
 
   Stdout._(IOSink sink, this._fd) : super(sink);
 
-  /**
-   * Returns true if there is a terminal attached to stdout.
-   */
+  /// Whether there is a terminal attached to stdout.
   bool get hasTerminal => _hasTerminal(_fd);
 
-  /**
-   * Get the number of columns of the terminal.
-   *
-   * If no terminal is attached to stdout, a [StdoutException] is thrown. See
-   * [hasTerminal] for more info.
-   */
+  /// The number of columns of the terminal.
+  ///
+  /// If no terminal is attached to stdout, a [StdoutException] is thrown. See
+  /// [hasTerminal] for more info.
   int get terminalColumns => _terminalColumns(_fd);
 
-  /**
-   * Get the number of lines of the terminal.
-   *
-   * If no terminal is attached to stdout, a [StdoutException] is thrown. See
-   * [hasTerminal] for more info.
-   */
+  /// The number of lines of the terminal.
+  ///
+  /// If no terminal is attached to stdout, a [StdoutException] is thrown. See
+  /// [hasTerminal] for more info.
   int get terminalLines => _terminalLines(_fd);
 
-  /**
-    * Whether connected to a terminal that supports ANSI escape sequences.
-    *
-    * Not all terminals are recognized, and not all recognized terminals can
-    * report whether they support ANSI escape sequences, so this value is a
-    * best-effort attempt at detecting the support.
-    *
-    * The actual escape sequence support may differ between terminals,
-    * with some terminals supporting more escape sequences than others,
-    * and some terminals even differing in behavior for the same escape
-    * sequence.
-    *
-    * The ANSI color selection is generally supported.
-    *
-    * Currently, a `TERM` environment variable containing the string `xterm`
-    * will be taken as evidence that ANSI escape sequences are supported.
-    * On Windows, only versions of Windows 10 after v.1511
-    * ("TH2", OS build 10586) will be detected as supporting the output of
-    * ANSI escape sequences, and only versions after v.1607 ("Anniversary
-    * Update", OS build 14393) will be detected as supporting the input of
-    * ANSI escape sequences.
-    */
+  /// Whether connected to a terminal that supports ANSI escape sequences.
+  ///
+  /// Not all terminals are recognized, and not all recognized terminals can
+  /// report whether they support ANSI escape sequences, so this value is a
+  /// best-effort attempt at detecting the support.
+  ///
+  /// The actual escape sequence support may differ between terminals,
+  /// with some terminals supporting more escape sequences than others,
+  /// and some terminals even differing in behavior for the same escape
+  /// sequence.
+  ///
+  /// The ANSI color selection is generally supported.
+  ///
+  /// Currently, a `TERM` environment variable containing the string `xterm`
+  /// will be taken as evidence that ANSI escape sequences are supported.
+  /// On Windows, only versions of Windows 10 after v.1511
+  /// ("TH2", OS build 10586) will be detected as supporting the output of
+  /// ANSI escape sequences, and only versions after v.1607 ("Anniversary
+  /// Update", OS build 14393) will be detected as supporting the input of
+  /// ANSI escape sequences.
   bool get supportsAnsiEscapes => _supportsAnsiEscapes(_fd);
 
   external bool _hasTerminal(int fd);
@@ -268,18 +237,19 @@ class Stdout extends _StdSink implements IOSink {
   external int _terminalLines(int fd);
   external static bool _supportsAnsiEscapes(int fd);
 
-  /**
-   * Get a non-blocking `IOSink`.
-   */
+  /// A non-blocking `IOSink` for the same output.
   IOSink get nonBlocking {
-    _nonBlocking ??= new IOSink(new _FileStreamConsumer.fromStdio(_fd));
-    return _nonBlocking;
+    return _nonBlocking ??= new IOSink(new _FileStreamConsumer.fromStdio(_fd));
   }
 }
 
+/// Exception thrown by some operations of [Stdout]
 class StdoutException implements IOException {
+  /// Message describing cause of the exception.
   final String message;
-  final OSError osError;
+
+  /// The underlying OS error, if available.
+  final OSError? osError;
 
   const StdoutException(this.message, [this.osError]);
 
@@ -288,9 +258,13 @@ class StdoutException implements IOException {
   }
 }
 
+/// Exception thrown by some operations of [Stdin]
 class StdinException implements IOException {
+  /// Message describing cause of the exception.
   final String message;
-  final OSError osError;
+
+  /// The underlying OS error, if available.
+  final OSError? osError;
 
   const StdinException(this.message, [this.osError]);
 
@@ -337,15 +311,15 @@ class _StdSink implements IOSink {
     _sink.encoding = encoding;
   }
 
-  void write(object) {
+  void write(Object? object) {
     _sink.write(object);
   }
 
-  void writeln([object = ""]) {
+  void writeln([Object? object = ""]) {
     _sink.writeln(object);
   }
 
-  void writeAll(objects, [sep = ""]) {
+  void writeAll(Iterable objects, [String sep = ""]) {
     _sink.writeAll(objects, sep);
   }
 
@@ -353,7 +327,7 @@ class _StdSink implements IOSink {
     _sink.add(data);
   }
 
-  void addError(error, [StackTrace stackTrace]) {
+  void addError(error, [StackTrace? stackTrace]) {
     _sink.addError(error, stackTrace);
   }
 
@@ -367,30 +341,21 @@ class _StdSink implements IOSink {
   Future get done => _sink.done;
 }
 
-/// The type of object a standard IO stream is attached to.
+/// The type of object a standard IO stream can be attached to.
 class StdioType {
   static const StdioType terminal = const StdioType._("terminal");
   static const StdioType pipe = const StdioType._("pipe");
   static const StdioType file = const StdioType._("file");
   static const StdioType other = const StdioType._("other");
 
-  @Deprecated("Use terminal instead")
-  static const StdioType TERMINAL = terminal;
-  @Deprecated("Use pipe instead")
-  static const StdioType PIPE = pipe;
-  @Deprecated("Use file instead")
-  static const StdioType FILE = file;
-  @Deprecated("Use other instead")
-  static const StdioType OTHER = other;
-
   final String name;
   const StdioType._(this.name);
   String toString() => "StdioType: $name";
 }
 
-Stdin _stdin;
-Stdout _stdout;
-Stdout _stderr;
+final Stdin _stdin = _StdIOUtils._getStdioInputStream(_stdinFD);
+final Stdout _stdout = _StdIOUtils._getStdioOutputStream(_stdoutFD);
+final Stdout _stderr = _StdIOUtils._getStdioOutputStream(_stderrFD);
 
 // These may be set to different values by the embedder by calling
 // _setStdioFDs when initializing dart:io.
@@ -407,8 +372,7 @@ void _setStdioFDs(int stdin, int stdout, int stderr) {
 
 /// The standard input stream of data read by this program.
 Stdin get stdin {
-  _stdin ??= _StdIOUtils._getStdioInputStream(_stdinFD);
-  return _stdin;
+  return IOOverrides.current?.stdin ?? _stdin;
 }
 
 /// The standard output stream of data written by this program.
@@ -417,8 +381,7 @@ Stdin get stdin {
 /// result in an unhandled asynchronous error unless there is an error handler
 /// on `done`.
 Stdout get stdout {
-  _stdout ??= _StdIOUtils._getStdioOutputStream(_stdoutFD);
-  return _stdout;
+  return IOOverrides.current?.stdout ?? _stdout;
 }
 
 /// The standard output stream of errors written by this program.
@@ -427,11 +390,10 @@ Stdout get stdout {
 /// result in an unhandled asynchronous error unless there is an error handler
 /// on `done`.
 Stdout get stderr {
-  _stderr ??= _StdIOUtils._getStdioOutputStream(_stderrFD);
-  return _stderr;
+  return IOOverrides.current?.stderr ?? _stderr;
 }
 
-/// For a stream, returns whether it is attached to a file, pipe, terminal, or
+/// Whether a stream is attached to a file, pipe, terminal, or
 /// something else.
 StdioType stdioType(object) {
   if (object is _StdStream) {
@@ -456,7 +418,7 @@ StdioType stdioType(object) {
     return StdioType.file;
   }
   if (object is Socket) {
-    int socketType = _StdIOUtils._socketType(object);
+    int? socketType = _StdIOUtils._socketType(object);
     if (socketType == null) return StdioType.other;
     switch (socketType) {
       case _stdioHandleTypeTerminal:
@@ -484,6 +446,6 @@ class _StdIOUtils {
   external static Stdin _getStdioInputStream(int fd);
 
   /// Returns the socket type or `null` if [socket] is not a builtin socket.
-  external static int _socketType(Socket socket);
+  external static int? _socketType(Socket socket);
   external static _getStdioHandleType(int fd);
 }
